@@ -12,6 +12,7 @@ class GeneralTaskModule(LightningModule):
         self.opt = optimizer
         self.metrics = metrics
         self.messages = dict()          # 存放训练、验证、测试过程中的各种消息数据
+        self.do_test_loss = True
 
     def forward(self, *batch_data):                         # 前向计算
         return self.model(*batch_data)
@@ -37,14 +38,18 @@ class GeneralTaskModule(LightningModule):
         return {'val_loss': loss}
 
     def test_step(self, batch, batch_nb):                   # 测试步
-        loss, preds, targets, batch_size = self.do_forward(batch)
-        self.log('test_loss', loss, prog_bar=True, on_step=True, on_epoch=True, batch_size=batch_size)
+        loss, preds, targets, batch_size = self.do_forward(batch, self.do_test_loss)
+        if loss is not None:
+            self.log('test_loss', loss, prog_bar=True, on_step=True, on_epoch=True, batch_size=batch_size)
 
         preds, targets = detach_clone(preds), detach_clone(targets)
         if self.metrics:
             self.do_metric(preds, targets, 'test', True, True)
         self.messages['test_batch'] = (batch_nb, preds, targets)  # (batch_idx, preds, tagets)
-        return {'test_loss': loss}
+        if loss is not None:
+            return {'test_loss': loss}
+        else:
+            return None
 
     def configure_optimizers(self):                         # 优化器
         if isinstance(self.opt, (list, tuple)) and len(self.opt) == 2 \
@@ -57,10 +62,13 @@ class GeneralTaskModule(LightningModule):
         else:
             return self.opt
 
-    def do_forward(self, batch):                            # 前向计算
+    def do_forward(self, batch, do_loss=True):                            # 前向计算
         input_feat, targets = batch[:-1], batch[-1]  # batch最后一个元素为标签
         preds = self(*input_feat)
-        loss = self.loss_fn(preds, targets)
+        if do_loss:
+            loss = self.loss_fn(preds, targets)
+        else:
+            loss = None
         batch_size = batch[0].shape[0] if isinstance(batch, (tuple, list)) else batch.shape[0]
         return loss, preds, targets, batch_size
 
