@@ -79,7 +79,7 @@ class Trainer(PLTrainer):
 
         super().__init__(**self.init_params)
 
-    def _prepare_metrics(self, metrics):
+    def _prepare_metrics(self, metrics, just_for_test=False):
         metrics_ready = {'train': [], 'val':[], 'test':[]}
         for m in metrics:
             if isinstance(m, (tuple, list)):  # when m is (metric_name, metric)
@@ -89,16 +89,22 @@ class Trainer(PLTrainer):
             if isinstance(m, (Metric, MetricBase)):
                 if not hasattr(m, 'name'):
                     m.name = m.__class__.__name__
-                metrics_ready['train'].append(m)
-                metrics_ready['val'].append(m.clone())
-                metrics_ready['test'].append(m.clone())
+                if just_for_test:
+                    metrics_ready['test'].append(m.clone())
+                else:
+                    metrics_ready['train'].append(m)
+                    metrics_ready['val'].append(m.clone())
+                    metrics_ready['test'].append(m.clone())
             else:
                 if not hasattr(m, 'name'):
                     name = m.__name__ if hasattr(m, '__name__') else type(m).__name__
-                metrics_ready['train'].append(MetricBase(m, name))
-                metrics_ready['val'].append(MetricBase(m, name))
-                metrics_ready['test'].append(MetricBase(m, name))
-        return metrics_ready
+                if just_for_test:
+                    metrics_ready['test'].append(MetricBase(m, name))
+                else:
+                    metrics_ready['train'].append(MetricBase(m, name))
+                    metrics_ready['val'].append(MetricBase(m, name))
+                    metrics_ready['test'].append(MetricBase(m, name))
+        return metrics_ready if not just_for_test else metrics_ready['test']
 
     def fit(self, train_dl=None, val_dl=None, epochs=None, ckpt_path=None):
         if self.datamodule:
@@ -107,7 +113,12 @@ class Trainer(PLTrainer):
             super().fit(self.task_module, train_dataloaders=train_dl, val_dataloaders=val_dl, ckpt_path=ckpt_path)
         return self.progress
 
-    def test(self, test_dl=None, ckpt_path='best', pl_module=None, verbose=True):
+    def test(self, test_dl=None, ckpt_path='best', pl_module=None, verbose=True, metrics=None, do_loss=True):
+        self.task_module.do_test_loss = do_loss
+        if metrics is not None:  # 如果提供了仅用于test的metrics
+            test_metrics = self._prepare_metrics(metrics, just_for_test=True)
+            self.task_module.metrics['test'] = test_metrics
+
         if test_dl is not None:
             return super().test(pl_module, dataloaders=test_dl, ckpt_path=ckpt_path, verbose=verbose)
         elif self.datamodule and self.datamodule.test_dataloader():
